package dev.shadowsoffire.apotheosis.adventure.compat;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.google.common.base.Preconditions;
import com.mojang.datafixers.kinds.App;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.serialization.Codec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import dev.shadowsoffire.apotheosis.adventure.compat.AdventureTwilightCompat;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;
import dev.shadowsoffire.placebo.color.GradientColor;
import java.util.Map;
import java.util.function.BiFunction;
import net.minecraft.ChatFormatting;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.network.chat.Style;
import net.minecraft.network.chat.TextColor;
import net.minecraft.util.Mth;
import net.minecraft.util.StringUtil;
import net.minecraft.world.effect.MobEffectInstance;
import net.minecraft.world.effect.MobEffects;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.entity.ai.attributes.Attributes;
import net.minecraft.world.item.ItemStack;
import twilightforest.entity.monster.Redcap;

public class TreasureGoblinBonus extends GemBonus {
  public static final Codec<TreasureGoblinBonus> CODEC;
  
  protected final Map<LootRarity, Data> values;
  
  static {
    CODEC = RecordCodecBuilder.create(inst -> inst.group(gemClass(), (App)LootRarity.mapCodec(Data.CODEC).fieldOf("values").forGetter(())).apply((Applicative)inst, dev.shadowsoffire.apotheosis.adventure.compat.AdventureTwilightCompat.TreasureGoblinBonus::new));
  }
  
  public TreasureGoblinBonus(GemClass gemClass, Map<LootRarity, Data> values) {
    super(Apotheosis.loc("twilight_treasure_goblin"), gemClass);
    this.values = values;
  }
  
  public void doPostAttack(ItemStack gem, LootRarity rarity, LivingEntity user, Entity target) {
    Data d = this.values.get(rarity);
    if (Affix.isOnCooldown(getCooldownId(gem), d.cooldown, user))
      return; 
    if (user.f_19796_.m_188501_() <= d.chance) {
      Redcap goblin = (Redcap)((EntityType)AdventureTwilightCompat.REDCAP.get()).m_20615_(user.m_9236_());
      CompoundTag tag = new CompoundTag();
      tag.m_128359_("DeathLootTable", "apotheosis:entity/treasure_goblin");
      goblin.m_7378_(tag);
      goblin.getPersistentData().m_128379_("apoth.treasure_goblin", true);
      goblin.m_6593_((Component)Component.m_237115_("name.apotheosis.treasure_goblin").m_130938_(s -> s.m_131148_((TextColor)GradientColor.RAINBOW)));
      goblin.m_20340_(true);
      goblin.m_21051_(Attributes.f_22279_).m_22125_(new AttributeModifier("apoth.very_fast", 0.2D, AttributeModifier.Operation.ADDITION));
      goblin.m_21051_(Attributes.f_22276_).m_22125_(new AttributeModifier("apoth.healmth", 60.0D, AttributeModifier.Operation.ADDITION));
      goblin.m_21153_(goblin.m_21233_());
      for (int i = 0; i < 8; i++) {
        int x = Mth.m_216271_(goblin.f_19796_, -5, 5);
        int y = Mth.m_216271_(goblin.f_19796_, -1, 1);
        int z = Mth.m_216271_(goblin.f_19796_, -5, 5);
        goblin.m_146884_(target.m_20182_().m_82520_(x, y, z));
        if (user.m_9236_().m_45786_((Entity)goblin))
          break; 
        if (i == 7)
          goblin.m_146884_(target.m_20182_()); 
      } 
      goblin.m_7292_(new MobEffectInstance(MobEffects.f_19619_, 200, 0));
      user.m_9236_().m_7967_((Entity)goblin);
      Affix.startCooldown(getCooldownId(gem), user);
    } 
  }
  
  public Codec<? extends GemBonus> getCodec() {
    return (Codec)CODEC;
  }
  
  public GemBonus validate() {
    Preconditions.checkArgument(!this.values.isEmpty(), "No values provided!");
    return this;
  }
  
  public boolean supports(LootRarity rarity) {
    return this.values.containsKey(rarity);
  }
  
  public int getNumberOfUUIDs() {
    return 0;
  }
  
  public Component getSocketBonusTooltip(ItemStack gem, LootRarity rarity) {
    Data d = this.values.get(rarity);
    MutableComponent mutableComponent = Component.m_237110_("affix.apotheosis.cooldown", new Object[] { StringUtil.m_14404_(d.cooldown) });
    return (Component)Component.m_237110_("bonus." + getId() + ".desc", new Object[] { Affix.fmt(d.chance * 100.0F), mutableComponent }).m_130940_(ChatFormatting.YELLOW);
  }
}
